from multiclass_classification import MulticlassClassification
from preprocessing import create_emoticons_tags, create_url_tags, create_time_tags, clean_texts, \
                                       stemming, oversampling, w2v, pipeline
